from abc import ABC, abstractmethod
from dosuby.src.core.domain.config import Config
from dosuby.src.core.domain.cache import Cache
from dosuby.src.interfaces.success_response import SuccessResponse
import logging
from dosuby.src.core.application.decorators.loggers_decorators import info_logger
from rich.console import Console
from rich.table import Table

class EnumerationStrategy(ABC):
    
    def __init__(self) -> None:
        super().__init__()
        self.config = Config()
    
    
    @abstractmethod
    def enumeration_process(self, subdomains_links) -> list:
        """This class manage the logic for enumeration

        Args:
            subdomains_links (Generator): link generator

        Returns:
            list: list of enumeration result
        """
        pass
    
    
    
    def display_result(self, result: dict):
        if self.config.scanning_modules:
            table = Table(title="\n")
            for col in result.get('columns'):
                table.add_column(col)
            for ro in result.get('row'):
                table.add_row(*ro,style='bright_green')
            
            console = Console()
            console.print(table)
    
    def display_result_count(self, succes_response: SuccessResponse):
        _ = succes_response.get_target_subdomains()
        cache = Cache()
        r_c = cache.cached_enumeration_result_count
        logging.info(f"{'-'*6}[SUBDOMAINS FOUND:    {r_c}]{'-'*6}")


from dosuby.src.interfaces.subdomain_enumerator_service import SubdomainEnumeratorService
from dosuby.src.core.application.use_cases.hackertarget_enumeration_use_case import HackertargetEnumerationUseCase
from dosuby.src.core.domain.target import Target
from dosuby.src.adapter.hackertarget.hackertarget_adapter import HackertargetAdapter
from dosuby.src.interfaces.success_response import SuccessResponse

from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.strategies.hackertarget.cli.hackertarget_cli_enumeration_strategy import HackertargetCliEnumerationStrategy

class HackertargetCliService(SubdomainEnumeratorService):
    
 
    def __init__(self, enumeration_strategy=HackertargetCliEnumerationStrategy(), success_response=SuccessResponse()) -> None:
        super().__init__(enumeration_strategy, success_response)
    
    
    def build_enumerator(self, target: Target):
        hackertarget_adapter = HackertargetAdapter()
        target_hackertarget_usecase = HackertargetEnumerationUseCase(hackertarget=hackertarget_adapter)
        result = target_hackertarget_usecase.execute(target=target)
        return result
    
    def process_enumerator(self, result) -> SuccessResponse:
        return self.enumeration_strategy.enumeration_process(subdomains_links=result, success_response=self.success_response)     
                                                            


import sys, os
from .search_response import SearchResponse
from dosuby.src.core.domain.target import Target

class FailedResponse(SearchResponse):
    def __init__(self) -> None:
        super().__init__()
        self._res_cause_of_failure = None
    
    @property
    def cause_of_failure(self):
        if self._res_cause_of_failure is None: return 'Unknown'
        if self._res_cause_of_failure['label'] is None: return 'Unkonwn'
        return self._res_cause_of_failure['label']
    
    @property
    def cause_of_failure_to_dict(self):
        return self._res_cause_of_failure
    
    @cause_of_failure.setter
    def cause_of_failure(self, value):
        self._res_cause_of_failure = value
        
    @property
    def response_type_to_dict(self):
        return self._res_type
    
    
    def _format_message(self, msg):
        if isinstance(msg, Exception):
            exc_type, exc_obj, exc_tb = sys.exc_info()
            if exc_tb is not None:
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                line_number = exc_tb.tb_lineno
                return "{}: {} in '{}' line {}".format(msg.__class__.__name__, "{}".format(msg), "{}".format(fname), "{}".format(line_number))
           
            return "{}: {} ".format(msg.__class__.__name__, "{}".format(msg))

        return msg

    
    def get_response(self):
        response = {
            'type': self.response_type,
            'status_code': self.status_code,
            'cause': self.cause_of_failure,
            'message': self._format_message(self.response_message),
        }
        
        return  f"Status:   {response.get('status_code')}\n"\
                f"{'-'*10}\n"\
                f"Type:     {response.get('type')}\n"\
                f"Message:  '{response.get('message')}'\n"\
                f"{'-'*10}\n"

    def __str__(self) -> str:
        return self.get_response()
    
    def set_target(self, target: Target):
        return super().set_target(target)


import pytest
from dosuby.src.core.application.input_dtos.target_input_dto import TargetInputDTO
from dosuby.src.core.application.use_cases.dorks_enumeration_use_case import DorksEnumerationUseCase
from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
from dosuby.src.adapter.dorks.google_dorks_adapter import GoogleDorksAdapter
from unittest import mock

fake_uri = 'sometext.test'


fake_subdomains = ['fake.sometext.test', 'dork.sometext.test']

def test_enumerate_using_invalid_dork_use_case():
    
    
    fake_dork = 'fake dork'
    
    with pytest.raises(ValueError):
        DorksEnumerationUseCase(dork=fake_dork)


# def test_enumerate_using_google_dorks_use_case(monkeypatch):
    
#     # Create a fake google dorks enumeration result
#     def mock_google_dork_get_results(self):
#         return fake_subdomains
    
#     monkeypatch.setattr(GoogleDorksAdapter, 'get_results', mock_google_dork_get_results)
    
#     fake_google_dork = GoogleDorksAdapter()
    
#     # Inject the fake GoogleDorksAdapter
#     target_subdomains_use_case = DorksEnumerationUseCase(dork=fake_google_dork)
#     # Set a target uri
#     target_input = TargetInputDTO(uri=fake_uri)
#     # Inject the target DTO input
#     target_subdomains_result = target_subdomains_use_case.execute(target=target_input)
    
#     assert target_subdomains_result == fake_subdomains

def test_invalid_target_dto_input_in_dorks_use_case_execute():
    
    invalid_target_input = 'Invalid target DTO'
    
    fake_dork = GoogleDorksAdapter()
    
    target_subdomains_use_case = DorksEnumerationUseCase(dork=fake_dork)
    
    with pytest.raises(ValueError):
        target_subdomains_use_case.execute(target=invalid_target_input)


import pytest
from dosuby.src.adapter.dorks.google_dorks_adapter import GoogleDorksAdapter
import unittest.mock


def test_google_dorks_are_set_correctly(monkeypatch):
    
    target_uri = 'faketarget.com'
    
    moke_queries = [f'site:*.{target_uri}', f'inurl:{target_uri}']
    
    monkeypatch.setattr(GoogleDorksAdapter, 'queries', moke_queries)
    
    google_dork = GoogleDorksAdapter()
    
    assert google_dork.queries == ['site:*.faketarget.com', 'inurl:faketarget.com']

def test_add_a_new_google_dork_query(monkeypatch):
    
    target_uri = 'faketarget.com'
    
    moke_queries = [f'site:*.{target_uri}', f'inurl:{target_uri}']
    
    monkeypatch.setattr(GoogleDorksAdapter, 'queries', moke_queries)
    
    google_dork = GoogleDorksAdapter()
    
    google_dork.add_dork_queries(query=f'{target_uri} test_query')
    
    assert google_dork.queries == ['site:*.faketarget.com', 'inurl:faketarget.com', 'faketarget.com test_query']
    


from dosuby.src.interfaces.security_enumeration import SecurityEnumeration

class WebserverEnumerationAdapter(SecurityEnumeration):
    
    def __init__(self) -> None:
        """This class will manage the port enumeration logic using the Sockets

        """
        super().__init__()
    
    
    def run(self):
        return self._process()
    

    def _process(self, **kwargs):
        return super()._process(**kwargs)
        


import pytest
from unittest import mock
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException


def test_invalid_target_input_error():
    
    req = mock.Mock()
    
    req.errors = [{'parameter': 'target', 'message': 'Invalid'}]
    
    ex = InvalidTargetException()
    
    ex.add_error('target', 'Invalid')
    
    assert req.errors == ex.errors
    
    assert bool(ex) is True
    
    with pytest.raises(Exception):
        ex.errors = [{'parameter': 'target', 'message': 'Invalid'}]

def test_invalid_target_input_error_string():
    
    req = mock.Mock()
    
    def mock_load_invalid_target():
        req.target = None
        if req.target is None:
            raise InvalidTargetException(error={'parameter': 'target', 'message': 'Invalid'})
        return 
        
    req.load_target.side_effect = mock_load_invalid_target
    
    with pytest.raises(InvalidTargetException, match="target: Invalid"):
        req.load_target()



#!./venv/bin/python3

import argparse
import sys
from dosuby.infrastructure.libs.helpers.dosuby_module import DosubyModule


parser = argparse.ArgumentParser(description='Dosuby manager')
action_group = parser.add_mutually_exclusive_group(required=True)
action_group.add_argument('--create', action='store_true', help='to create a new enumeration module')
action_group.add_argument('--list', action='store_true', help='to list enumeration modules')
action_group.add_argument('--delete', action='store_true', help='to delete an enumeration module')
parser.add_argument('--name', help='the name of the enumeration module to create or delete')

args = parser.parse_args()

if args.create and not args.name:
    parser.error('--name is required when --create is specified')
elif not args.list and not args.create and not args.delete:
    parser.error('one of --create, --list, or --delete is required')

if args.create:
    print(f'Creating {args.name} module...')
    enumeration_module = DosubyModule(class_name=args.name)
    enumeration_module.create()
elif args.list:
    print(f'List of enumeration modules')
    DosubyModule.list_modules()
elif args.delete:
    confirmation = input(f'Are you sure you want to delete {args.name} module? (y/N): ')
    if confirmation.lower() == 'y':
        print(f'Deleting {args.name} module...')
        enumeration_module = DosubyModule(class_name=args.name)
        enumeration_module.delete()
    else:
        print("[*] Deletion cancelled")
        sys.exit(0)



from dosuby.src.interfaces.security_enumeration import SecurityEnumeration

class CMSEnumerationAdapter(SecurityEnumeration):
    
    def __init__(self) -> None:
        """This class will manage the port enumeration logic using the Sockets

        """
        super().__init__()
        self.subdomain_uri = ''
        self.user_agent = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.116 Safari/537.36',
            }
        
    
    
    def run(self):
        yield self._process()
    

    def _process(self, **kwargs):
        return super()._process(**kwargs)
    
    def scann_for_cms_by_url(self, ip, index):
        """Run the CMS scan logic for the given IP using url

        Args:
            ip (str): Subdomain IP
            index (str): page name related to a cms
        """
        NotImplementedError("scann_for_cms is not implemented")


from dosuby.src.adapter.crt.crt_adapter import CrtAdapter
from dosuby.infrastructure.libs.CrtSearch.crt_search import CrtSearch


# def test_crt_search_return_correct_data():
#     # crt_adapter = CrtAdapter()
#     crt_search = CrtSearch()
#     rs = crt_search.search(query='uca.ma')
#     data = []
#     i = 0
#     for r in rs:
#         cells = r.find_all("td")
#         # Check if there are at least 5 cells in the row
#         if len(cells) >= 5:
#             fifth_cell = cells[4]
#             data.append(fifth_cell.text)
    
#     assert data[1] == 'yyy.este.uca.ma'
#     assert data[4] == 'ecampus-cloud.uca.ma'
#     assert data[11] == 'rh-ensas.uca.ma'
    



from .search_response import SearchResponse
from dosuby.src.core.application.enums.cli_response_type_enum import CliResponseTypeEnums
from dosuby.src.core.domain.target import Target
from dosuby.src.core.domain.config import Config
from dosuby.src.core.application.decorators.loggers_decorators import *
from dosuby.src.core.application.decorators.loggers_decorators import Loader
from rich.console import Console
from rich.table import Table


class SuccessResponse(SearchResponse):
    """Successful search result.
    """
    
    def __init__(self) -> None:
        super().__init__()
        self.status_code = CliResponseTypeEnums.SUCCESS['status_code']
        self.response_type = CliResponseTypeEnums.SUCCESS
        self.config = Config()
        self.loader = Loader()
    
    def get_response(self):
        if self.config.scanning_modules is True:
            return self._get_response_with_scanning_modules()
        
        return None
        # return self._get_response_without_scanning_modules()
            
            
    def get_target_subdomains(self):
        if self.target:
            return self.target.get_target_intel()
        return []
    
    def get_target_subdomains_count(self) -> int:
        if self.target:
            return self.target.subdomains_count
        return 0
    
    def set_target(self, target: Target):
        return super().set_target(target)
    
    
    def _get_response_with_scanning_modules(self):
        response = {
            'type': self.response_type,
            'status_code': self.status_code,
            'message': self.response_message,
            'subdomain': self.target.subdomain.subdomain_uri,
            'subdomain_ip': self.target.subdomain.subdomain_ip,
            'subdomain_ports': self.target.subdomain.subdomain_open_ports_from_uri,
            'subdomain_cms': self.target.subdomain.subdomain_cms,
            'subdomain_webserver': self.target.subdomain.subdomain_webserver,
        }
        
        if not response.get('subdomain_ip'):
            ip_str = 'N/A'
        else:
            ip_str = response.get('subdomain_ip')
        
        if not response.get('subdomain_ports'):
            open_ports_str = 'N/A'
        else:
            open_ports_str = ", ".join([str(num) for num in response.get('subdomain_ports')])
        
        if not response.get('subdomain_cms'):
            cms_str = 'N/A'
        else:
            cms_str = response.get('subdomain_cms')
        
        if not response.get('subdomain_webserver'):
            webserver_str = 'N/A'
        else:
            webserver_str = response.get('subdomain_webserver')
            
        
        columns = ["Subdomain", "IP", "Open Ports", "CMS", "Web Server"]
        row = [
            [
                response.get('subdomain'),
                ip_str,
                open_ports_str,
                cms_str,
                webserver_str
            ]
        ]
        
        # return f"\n{'-'*20}\n"\
        #     f"{G}--> {response.get('subdomain')}{G}\n"\
        #     f"{W}   [+] IP: {response.get('subdomain_ip')}{W}\n"\
        #     f"{W}   [+] Ports: {response.get('subdomain_ports')}{W}\n"\
        #     f"{W}   [+] CMS: {response.get('subdomain_cms')}{W}\n"\
        #     f"{W}   [+] Webserver: {response.get('subdomain_webserver')}{W}\n"\
        #     f"{'-'*20}\n"
        return {
            'columns': columns,
            'row': row
        }
    
    def _get_response_without_scanning_modules(self):
        response = {
            'type': self.response_type,
            'status_code': self.status_code,
            'message': self.response_message,
            'subdomain': self.target.subdomain.subdomain_uri,
        }
    
        return f"\n{'-'*20}\n"\
            f"{C}--> {response.get('subdomain')}{C}\n"\
            f"{'-'*20}\n"
    
    


from typing import Union
import re

class ValidateTargetInput:
    """Validate the user input before starting the subdomains enumeration
    """
    
    @classmethod
    def extract_domain(self, target_uri: str) -> Union[str, bool] :
        """Extract the domain name from target uri.\n
        if target is not valid return false

        Args:
            target (str): target string (ex: http://www.example.com, https://www.example.com, ftp://www.example.com, exemple.come)

        Returns:
            str: target domain name
            bool: False if target is not valid
        """
        pattern = re.compile(r'(?:https?://)?(?:www\.)?([\w-]+\.[\w-]+)')
        
        match = pattern.search(target_uri)
        
        return match.group(1) if match else False


import requests
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
from dosuby.src.interfaces.cms_enumeration_adapter import CMSEnumerationAdapter

class WordPressScanningAdapter(CMSEnumerationAdapter):
    
    
    
    
    def run(self):
        if self.subdomain_uri:
            return self._process(uri=self.subdomain_uri)
        return None
    

    def _process(self, uri: str) -> str | None:
        wpLoginCheck = self.scann_for_cms_by_url(uri, '/wp-login.php')
        if wpLoginCheck.status_code == 200 and "user_login" in wpLoginCheck.text and "404" not in wpLoginCheck.text:
            return "Potential WordPress WP-Login page"

        wpAdminCheck = self.scann_for_cms_by_url(uri, '/wp-admin')
        if wpAdminCheck.status_code == 200 and "user_login" in wpAdminCheck.text and "404" not in wpAdminCheck.text:
            return "Potential WordPress WP-Admin page"
    
        wpAdminUpgradeCheck = self.scann_for_cms_by_url(uri, '/wp-admin/upgrade.php')
        if wpAdminCheck.status_code == 200 and "user_login" in wpAdminUpgradeCheck.text and "404" not in wpAdminUpgradeCheck.text:
            return "Potential WordPress WP-Admin page"
    
        wpLinksCheck = self.scann_for_cms_by_url(uri, '/')
        if 'wp-' in wpLinksCheck.text:
            return "Potential WordPress wp- style links detected on index"
    
        wpAdminReadMeCheck = self.scann_for_cms_by_url(uri, '/readme.html')
        if wpAdminReadMeCheck.status_code == 200 and "404" not in wpAdminReadMeCheck.text:
            return "Potential WordPress Readme.html"
        
        return None

    def scann_for_cms_by_url(self, uri, index):
        return requests.get(f"http://{uri}{index}", allow_redirects=False, headers=self.user_agent, verify=False)


from dosuby.src.core.domain.config import Config
from dosuby.src.handlers.cli.google_dork_handler import GoogleDorkHandler
from dosuby.src.handlers.cli.yahoo_dork_handler import YahooDorkHandler
from dosuby.src.handlers.cli.duckduckgo_dork_handler import DuckduckgoDorkHandler
from dosuby.src.handlers.cli.brave_dork_handler import BraveDorkHandler
from dosuby.src.handlers.cli.aol_dork_handler import AolDorkHandler
from dosuby.src.handlers.cli.crt_search_handler import CrtSearchHandler
from dosuby.src.handlers.cli.bing_dork_handler import BingDorkHandler
from dosuby.src.handlers.cli.anubis_dork_handler import AnubisHandler
from dosuby.src.handlers.cli.ask_dork_handler import AskDorkHandler
from dosuby.src.handlers.cli.alientvault_handler import AlientvaultHandler
from dosuby.src.handlers.cli.hackertarget_handler import HackertargetHandler
from dosuby.src.handlers.cli.waybackmachine_handler import WaybackmachineHandler
from dosuby.src.handlers.cli.virustotal_handler import VirustotalHandler
from dosuby.src.interfaces.handler import Handler
from dosuby.src.repositories.cli_report_repo import CliReportRepo


try:
    from dosuby.infrastructure.libs.Search_Engines_Scraper.search_engines import Google
except ImportError:
    print("No module named 'search_engines' found")
    
from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
from time import sleep

class GoogleDorksAdapter(SubdomainEnumerator):
    
    def __init__(self) -> None:
        """This class will manage the enumeration logic using the Google dorks

        """
        super().__init__()
        self.engine = Google()
    
    @property
    def queries(self):
        """Default GoogleDorkAdapter queries

        Returns:
            list: returns a list of google dorks queries to enumerate subdomains\n
            for a given target uri
        """
        return [
            f'site:{self.target_uri} -www',
            f'site:{self.target_uri} -www',
            f'site:*.{self.target_uri}  -www',
            f'site:*.*.{self.target_uri}  -www',
            f'site:*.*.*{self.target_uri}  -www',
        ]
    
    def add_dork_queries(self, query: str) -> None:
        """Add a new Google dork query

        Args:
            query (str): should be a google dork\nex: f'inurl:{target_uri}'

        Returns:
            _type_: None
        """
        return super().add_dork_queries(query)
    

    def _process(self, query):
        for page in self.engine.search(query=query,pages=1):
            for p in page:
                yield p
        


from dosuby.src.core.domain.cache import Cache


# def test_cache():
#     cache = Cache()
#     # print(cache.cached_subdomains)
#     cached_result = cache.check_if_ip_already_found_and_return_result(ip=value.get('ip'))
#     if cached_result:
#         return cached_result.get('open_ports')


from .handler_cli import HandlerCli
from dosuby.src.services.dorks.cli.aol_dork_cli_service import AolDorkCliService
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.response.cli.failed_response_builder import FailureResponseBuilder

class AolDorkHandler(HandlerCli):
    
    
    def run_service(self, uri, success_response: SuccessResponse):
        try:
            return AolDorkCliService(success_response=success_response).read(uri=uri)
        except Exception as ex:
            failed_response = FailureResponseBuilder().build_invalid_request_exception_object(ex)
            print(failed_response) 
    
    def __str__(self) -> str:
        return 'Using Aol Dork'
        
            


from dosuby.src.core.domain.target import Target
from dosuby.src.interfaces.security_enumeration import SecurityEnumeration

class PortsEnumerationUseCase:
    """This is the Ports Enumeration Use Case.\n
    it will be used to enumerate open ports of a subdomain
    """
    
    def __init__(self, security_enumerator: SecurityEnumeration) -> None:
        """Inject a search engine dork to be used

        Args:
            security_enumerator (SecurityEnumeration): security_enumerator .Must implement SecurityEnumeration interface
        """
        
        if not isinstance(security_enumerator, SecurityEnumeration):
            raise ValueError(f'security_enumerator param should be a SecurityEnumeration instance, not a {type(security_enumerator)}')
        
        self._security_enumerator = security_enumerator
    
    def execute(self, target: Target):
        """Execute method will run the enumeration process

        Args:
            target (Target): url or domain to enumerate
        """
        if not isinstance(target, Target):
            raise ValueError(f'target param should be a Target instance, not a {type(target)}.')
        self._security_enumerator.target_uri = target.target_uri.uri
        
        return self._security_enumerator.run()
    


from dataclasses import dataclass, field
from dosuby.src.interfaces.singleton import Singleton



class Config(metaclass=Singleton):
    """Dosuby configuration

    """
    
    scanning_modules: bool = field(init=True, default=False)


from dosuby.src.interfaces.enumeration_strategy import EnumerationStrategy

class WebserverCliEnumerationStrategy(EnumerationStrategy):
    
    
    def enumeration_process(self, **kwargs):
        """_summary_

        Args:
            result(Any): result  to process
            success_response (SuccessResponse): this will be used to get the final success response
        Returns:
            list: results list
        """
        # success response for each handler
        webserver = kwargs.get('webserver')
       
        return webserver


from abc import ABC, abstractmethod
from dosuby.src.interfaces.success_response import SuccessResponse

class EnumerationHandler(ABC):
    """
    The Handler interface declares a method for building the chain of subdomain handlers.
    """


    @abstractmethod
    def handle(self, uri: str, success_response: SuccessResponse=SuccessResponse()):
        pass
    
    @abstractmethod
    def run_service(self, uri: str, success_response: SuccessResponse):
        """This method will run the enumerator service for each service handler

        Args:
            success_response (SuccessResponse): success response (will be passed to the next handler)

        Raises:
            NotImplementedError: Method should be implemented by all Handlers
        """
        raise NotImplementedError



import pytest
from dosuby.src.core.application.validators.validate_target_input import ValidateTargetInput


def test_if_target_is_a_domain():
    
    fake_target_url = 'https://www.example.com'
    
    target_extracted_domain = ValidateTargetInput.extract_domain(target_uri=fake_target_url)
    
    assert target_extracted_domain == 'example.com'

def test_if_target_is_invalid():
    
    fake_target_url = "not-a-domain"
    
    target_extracted_domain = ValidateTargetInput.extract_domain(target_uri=fake_target_url)
    
    assert target_extracted_domain is False


from dosuby.src.interfaces.handler import Handler
import logging
from dosuby.src.core.application.decorators.loggers_decorators import *


class HandlerCli(Handler):
    """Handler Interface for Cli implementation

    Args:
        Handler (EnumerationHandler): Abstract class for enumerator handlers
    """
    
    def pre_handler_process(self, **kwargs):
        # Display handler name on the CLI
        # print('+'*20)
        # print(self)
        # print('+'*20)
        logging.info(f"{C}[*] Running enumeration{C} {W}[{self}]{W}")


from dosuby.src.interfaces.report_repo import ReportRepo
from dosuby.src.core.domain.enumeration_reporte import EnumerationReporte
from rich.console import Console
from rich.table import Table


class CliReportRepo(ReportRepo):

    def read_report(self):
        report = EnumerationReporte()
        report_subdomains = CliReportRepo.remove_duplicates(
            report.report_subdomains)
        table = Table(title="\nSubdomains Found")
        if not self.config.scanning_modules:
            columns = ["Subdomain", "IP"]
            for col in columns:
                table.add_column(col)

            subdomain: dict
            # print(report_subdomains)
            for subdomain in report_subdomains:
                if not subdomain.get('subdomain_ip'):
                    ip_str = 'N/A'
                else:
                    ip_str = subdomain.get('subdomain_ip')

                row = [
                    subdomain.get('subdomain_uri'),
                    ip_str,
                ]
                table.add_row(*row, style='bright_green')

            console = Console()
            console.print(table)
            return

        columns = ["Subdomain", "IP", "Open Ports", "CMS", "Web Server"]

        for col in columns:
            table.add_column(col)

        for subdomain in report_subdomains:
            if not subdomain.get('subdomain_ip'):
                ip_str = 'N/A'
            else:
                ip_str = subdomain.get('subdomain_ip')

            if not subdomain.get('subdomain_open_ports'):
                open_ports_str = 'N/A'
            else:
                open_ports_str = ", ".join(
                    [str(num) for num in subdomain.get('subdomain_open_ports')])

            if not subdomain.get('subdomian_cms'):
                cms_str = 'N/A'
            else:
                cms_str = subdomain.get('subdomian_cms')

            if not subdomain.get('subdomain_webserver'):
                webserver_str = 'N/A'
            else:
                webserver_str = subdomain.get('subdomain_webserver')

            row = [
                subdomain.get('subdomain_uri'),
                ip_str,
                open_ports_str,
                cms_str,
                webserver_str
            ]

            table.add_row(*row, style='bright_green')

        # for row in rows:
        #     table.add_row(*row,style='bright_green')

        console = Console()
        console.print(table)

    @staticmethod
    def remove_duplicates(data_list):
        unique_list = []
        for item in data_list:
            if item not in unique_list:
                unique_list.append(item)
        return unique_list



from dosuby.src.interfaces.response_builder import ResponseBuilder
from dosuby.src.interfaces.failed_response import FailedResponse
from dosuby.src.core.application.enums.cli_response_type_enum import CliResponseTypeEnums
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException

class FailureResponseBuilder(ResponseBuilder):

    def __init__(self) -> None:
        """
        A builder instance should contain a blank response object
        which is used in further assembly
        """
        self._failure_response = FailedResponse()
    
    def set_value(self, value):
        self._failure_response.response_value = value
        return self
    
    def set_type(self, value=None):
        self._failure_response.response_type = value
        return self
    
    def default_type(self):
        self._failure_response.response_type = CliResponseTypeEnums.ERROR
        return self
    
    def set_response_message(self, message_value):
        self._failure_response.response_message = message_value
        return self

    def build_response(self):
        self._failure_response.response_value = {
            'type': self._failure_response.response_type,
            'cause':  self._failure_response.cause_of_failure,
            'message': self._failure_response.response_message
        }
        
        
        return self._failure_response
    
    def _set_status_code(self, response_failed_type):
        self._failure_response.status_code = response_failed_type['status_code']
        return self
    
    def set_response_message_and_build(self, message_value=''):
        failure_builder = self.default_type().set_response_message(message_value)
        return failure_builder.build_response()
    
    def _set_cause_of_failure(self, value=CliResponseTypeEnums.PARAMETERS_ERROR):
        self._failure_response.cause_of_failure = value
        self = self._set_status_code(value)
        return self
        
    def build_invalid_request_exception_object(self, exc: InvalidTargetException):
        #default value 'PARAMETERS_ERROR'
        failure_response = self._set_cause_of_failure()\
                              .set_response_message_and_build(exc)
                             
        return failure_response

    def build_resource_error(self, message_value=''):
        failure_response = self._set_cause_of_failure(value=CliResponseTypeEnums.RESOURCE_ERROR)\
                              .set_response_message_and_build(message_value)
        
        return failure_response
    
    def build_parameters_error(self, message_value=''):
        failure_response = self._set_cause_of_failure()\
                            .set_response_message_and_build(message_value)
    
        return failure_response

    def build_system_error(self, message_value=''):
        RESP_TYPE = CliResponseTypeEnums.SYSTEM_ERROR
        failure_response = self._set_cause_of_failure(value=RESP_TYPE)\
                            ._set_status_code(response_failed_type=RESP_TYPE)\
                            .set_response_message_and_build(message_value)

        return failure_response


from dosuby.src.interfaces.subdomain_enumerator_service import SubdomainEnumeratorService
from dosuby.src.core.application.use_cases.dorks_enumeration_use_case import DorksEnumerationUseCase
from dosuby.src.core.domain.target import Target
from dosuby.src.adapter.dorks.brave_dorks_adapter import BraveDorksAdapter
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.response.cli.success_response_builder import SuccessResponseBuilder
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.strategies.dorks.cli.default_dork_cli_enumeration_strategy import DefaultDorkCliEnumerationStrategy

class BraveDorkCliService(SubdomainEnumeratorService):
    
 
    def __init__(self, enumeration_strategy=DefaultDorkCliEnumerationStrategy(), success_response=SuccessResponse()) -> None:
        super().__init__(enumeration_strategy, success_response)
    
    
    def build_enumerator(self, target: Target):
        brave_dork = BraveDorksAdapter()
        target_brave_dork_usecase = DorksEnumerationUseCase(dork=brave_dork)
        result = target_brave_dork_usecase.execute(target=target)
        return result
    
    def process_enumerator(self, result) -> SuccessResponse:
        return self.enumeration_strategy.enumeration_process(subdomains_links=result, success_response=self.success_response)     
                                                            


from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
from bs4 import BeautifulSoup
import json
import requests

class VirustotalAdapter(SubdomainEnumerator):
    
    def __init__(self) -> None:
        """Manage the enumeration logic using Virustotal

        """
        super().__init__()
        self.base_url = "https://www.virustotal.com/ui/domains/"
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
            'Accept-Encoding': 'gzip',
            'Accept-Ianguage': 'en-US,en;q=0.8', # Ianguage not Language
            'X-Tool': 'vt-ui-main',
            'X-VT-Anti-Abuse-Header': 'purple monkey dishwasher'
        }
    
    @property
    def queries(self):
        """Default VirustotalAdapter queries

        Returns:
            list: returns a list of Virustotal  queries to enumerate subdomains\n
            for a given target uri
        """
        return f"{self.base_url}{self.target_uri}/subdomains?relationships=resolutions"
    
    def add_dork_queries(self, query: str) -> None:
        """Add a new Anubis dork query

        Args:
            query (str): should be a google dork\nex: f'inurl:{target_uri}'

        Returns:
            _type_: None
        """
        return super().add_dork_queries(query)
    

    def _process(self, query):
        # write your process logic here
        response = requests.get(self.queries, headers=self.headers, proxies=None, verify=False)
        
        if response.status_code == 200:
            try:
                soup = BeautifulSoup(response.text, "lxml")
                virus_total_json = json.loads(soup.text)
                for i in virus_total_json['data']:
                    if i['type'] == 'domain':
                        subdomain = i['id']
                        yield subdomain
                for i in virus_total_json['data']:
                    for domain in i['attributes']['last_https_certificate']['extensions']['subject_alternative_name']:
                        domain = domain.lstrip('*.')
                        yield domain
            except:
                pass
        
        return []
            
        


import pytest
from dosuby.src.core.application.input_dtos.target_input_dto import TargetInputDTO
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException


def test_target_uri_existes():
    
    fake_uri = 'sometext.test'
    fake_target = TargetInputDTO(uri=fake_uri)
    
    assert fake_target.uri == fake_uri

def test_invalid_uri_input():
    
    invalid_fake_uri = 'invalid-uri-target'
    
    with pytest.raises(InvalidTargetException):
        fake_target = TargetInputDTO(uri=invalid_fake_uri)

def test_invalid_uri_with_correct_message():
    
    invalid_fake_uri = 'invalid-uri-target'
    
    with pytest.raises(InvalidTargetException, match="uri: Invalid target uri"):
        fake_target = TargetInputDTO(uri=invalid_fake_uri)
        


from dosuby.src.adapter.ports_scanning.socket_port_scanning_adapter import SocketPortScanningAdapter




# def test_socket_ports():
    
#     socket_adapter = SocketPortScanningAdapter()
#     assert socket_adapter.ports == [80]

# def test_socket_scanning_of_a_domain_list():
#     targets = ['cab.algerac.dz', 'mail.algerac.dz']
#     ports = []
    
#     port_scanning = SocketPortScanningAdapter()
#     for target in targets:
#         port_scanning.target_uri = target
#         open_ports = port_scanning.run()
#         ports.append(open_ports)
    
#     print(ports)


from abc import ABC, abstractmethod
from dosuby.src.core.application.decorators.loggers_decorators import info_logger

class SubdomainEnumerator(ABC):
    
    def __init__(self) -> None:
        """This class will manage the enumeration logic using an Adapter (ex: GoogleDorksAdapter) dorks

        """
        super().__init__()
        self._queries = []
        self._target_uri = ''
        self._extra_params = {}
        self._engine = None
    
    
    @property
    def target_uri(self):
        return self._target_uri
    
    @target_uri.setter
    def target_uri(self, value):
        self._target_uri = value
        
    @property
    def queries(self):
        return self._queries
    
    @queries.setter
    def queries(self, value):
        pass
        
    @property
    def engine(self):
        return self._engine

    @engine.setter
    def engine(self, value):
        self._engine = value
    
    
    @abstractmethod
    def add_dork_queries(self, query: str) -> None:
        self.queries.append(query)
    
    # @info_logger("Starting subdomains enumeration...")
    def run(self):
        """This method will run the dork enumeration process

        """
        for query in self.queries:
            yield self._process(query=query)
    
    
    @abstractmethod
    def _process(self, **kwargs):
        """This will run the dork enumeration logic

        Args:
            search_engine (object): search engine to be used
        """
        pass


class CliResponseTypeEnums:
    SUCCESS = { 'label': 'Success', 'value': True, 'status_code': 'OK'}
    SYSTEM_ERROR = { 'label': 'SystemError', 'value': False, 'status_code': 'FAILED'}
    PARAMETERS_ERROR = { 'label': 'ParametersError', 'value': False, 'status_code': 'FAILED'}
    RESOURCE_ERROR = { 'label': 'ResourceError', 'value': False, 'status_code': 'FAILED'}
    ERROR = { 'label': 'ERROR EXCEPTION', 'value': False, 'status_code': 'FAILED'}


import requests
import re
from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
from time import sleep

class BingDorksAdapter(SubdomainEnumerator):
    
    def __init__(self) -> None:
        """This class will manage the enumeration logic using the Bing dorks

        """
        super().__init__()
        self.base_url = "https://www.bing.com/search"
        self.headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36"}
    
    @property
    def queries(self):
        """Default BingDorkAdapter queries

        Returns:
            dict: returns a dict of params that will be used to enumerate subdomains\n
            for a given target uri
        """
        return {"q": "domain:." + self.target_uri, "pn": self.target_uri}
    
    def add_dork_queries(self, query: str) -> None:
        """Add a new Bing dork query

        Args:
            query (str): should be a google dork\nex: f'inurl:{target_uri}'

        Returns:
            _type_: None
        """
        return super().add_dork_queries(query)
    

    def _process(self, query):
        # Loop through Bing's search pages, 10 results at a time
        for page in range(1, 200, 10):
            params = self.queries
            params['first'] = str(page)
            response = requests.get(url=self.base_url, headers=self.headers, params=params, proxies=None, verify=False)
            if response.status_code == 200:
                try:
                    if "There are no results for" in response.text:
                        return []
                    else:
                        # Use regex to find all links on the page
                        links = re.findall(r'<a href="(http[s]?://[^"]+)"', response.text)
                        subdomains = [link for link in links if link.startswith("http") and self.target_uri in link]
                        for subdomain in subdomains:
                            yield subdomain
                except:
                    pass
            
            return []
                        


from dosuby.src.interfaces.security_enumerator_service import SecurityEnumeratorService
from dosuby.src.core.application.use_cases.cms_enumeration_use_case import CMSEnumerationUseCase
from dosuby.src.core.domain.subdomain import Subdomain
from dosuby.src.adapter.cms_scanning.wordpress_scanning_adapter import WordPressScanningAdapter
from dosuby.src.interfaces.success_response import SuccessResponse

from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.security_enumeration.cli.cms_cli_enumeration_strategy import CMSCliEnumerationStrategy

class WordPressScanningCliService(SecurityEnumeratorService):
    
 
    def __init__(self, enumeration_strategy=CMSCliEnumerationStrategy(), success_response=SuccessResponse()) -> None:
        super().__init__(enumeration_strategy, success_response)
    
    
    def build_enumerator(self, target: Subdomain) -> list:
        cms_scanning_adapter = WordPressScanningAdapter()
        cms_scanning_usecase = CMSEnumerationUseCase(security_enumerator=cms_scanning_adapter)
        result = cms_scanning_usecase.execute(target=target)
        return result
    
    def process_enumerator(self, result) -> SuccessResponse:
        return self.enumeration_strategy.enumeration_process(cms=result, success_response=self.success_response)     
                                                            


from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer



def test_serializer_value():
    
    ser_uri = ExtractUriSerializer.serialize(uri='domain.com')
    
    assert ser_uri == 'domain.com'


import socket
from dosuby.src.adapter.ports_scanning.socket_port_scanning_adapter import SocketPortScanningAdapter
from dosuby.src.core.domain.cache import Cache
from dosuby.src.core.domain.config import Config
from dosuby.src.core.application.enums.modules_status import ModuleStatus
from dosuby.src.core.domain.enumeration_reporte import EnumerationReporte
from dosuby.src.adapter.cms_scanning.wordpress_scanning_adapter import WordPressScanningAdapter
from dosuby.src.adapter.webserver_scanning.http_client_webserver_scanning_adapter import HttpClientWebserverScanningAdapter
from .loggers_decorators import *

SKIP_LOADING = False

def get_ip(func):
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        config = Config()
        # if not config.scanning_modules:
        #     return ModuleStatus.ABORT

        try:
            value = socket.gethostbyname(value)
        except:
            value = None
        
        return value
    return wrapper

def get_hostname(func):
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        config = Config()
        if not config.scanning_modules:
            return ModuleStatus.ABORT
        try:
            value = socket.gethostbyaddr(value)[0]
        except socket.error:
            value = "No reverse DNS record found"
        
        return value
    return wrapper

def get_open_ports(func):
    """Scan for open ports

    Args:
        func (Any): function that raturn a domain name
    """
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        config = Config()
        if not config.scanning_modules:
            return ModuleStatus.ABORT
        # check if the ip address is already in the cache
        cache = Cache()
        cached_result = cache.check_if_ip_already_found_and_return_result(ip=value.get('ip'))
        display_sub_info = True
        if cached_result:
            c_r = cached_result.get('open_ports')
            display_sub_info = True
            return c_r
        
        cache.add_subdomain_uri(value.get('uri'))
        if display_sub_info:
            logging.info(f"{G}[*]  ==> {value.get('uri')}{G}")
        
        loader = Loader(f"{Y}       [->] Ports Scanning...{Y}").start()
        loader.end = f"{Y}       [*] Ports Scanning{Y}{G} [DONE]{G}"
    
        
        try:
            port_scanning = SocketPortScanningAdapter()
            port_scanning.target_uri = value.get('uri')
            ports = port_scanning.run()
            loader.stop()
            return ports
        except:
            pass
        
        loader.stop()
        return []
    return wrapper

def add_to_list(attr_name):
    """values in a list

    Args:
        attr_name (list): list attribute to append to
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            setattr(args[0], attr_name, result)
            return result
        return wrapper
    return decorator

def cache_subdomain(func):
    def wrapper(self, value):
        try:
            cache_singleton = Cache()
            cache_singleton.add(value.get_cached_data())
        except Exception as exc:
            raise exc
        
        func(self, value)
    return wrapper

def save_enumeration_report(func):
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        
        try:
            report_singleton = EnumerationReporte()
            report_singleton.add(value)
        except:
            pass
        
        return value
    return wrapper

def scan_for_cms(func):
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        config = Config()
        if not config.scanning_modules:
            return ModuleStatus.ABORT
        
        try:
            cms = None
            loader = None
            cache_singleton = Cache()
            cached_result = cache_singleton.check_if_ip_already_found_and_return_result(ip=value.get('ip'))
            cached_uri = cache_singleton.check_if_uri_already_found_and_return_result(value.get('uri'))
            if 80 in cached_result.get('open_ports'):
                if cached_uri:
                    loader = Loader(f"{Y}       [->] CMS Scanning...{Y}").start()
                    loader.end = f"{Y}       [*] CMS Scanning{Y}{G} [DONE]{G}"
                cms_scanning_adapter = WordPressScanningAdapter()
                cms_scanning_adapter.subdomain_uri = value.get('uri')
                cms = cms_scanning_adapter.run()
            elif cached_uri:
                loader = Loader('').start()
                loader.end = f"{Y}       [*] CMS Scanning (No port 80){Y}{G} [SKIPED]{G}"
        except:
            cms = None
            
        if loader:
            loader.stop()
        return cms
    return wrapper

def save_cms(attr_name):
    """values in a list

    Args:
        attr_name (cms): cms attribute to append to
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            config = Config()
            if not config.scanning_modules:
                return ModuleStatus.ABORT
            cms = func(*args, **kwargs)
            setattr(args[0], attr_name, cms)
            return cms
        return wrapper
    return decorator

def save_webserver(attr_name):
    """values in a list

    Args:
        attr_name (webserver): webserver attribute to append to
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            config = Config()
            if not config.scanning_modules:
                return ModuleStatus.ABORT
            webserver = func(*args, **kwargs)
            setattr(args[0], attr_name, webserver)
            return webserver
        return wrapper
    return decorator

def get_webserver(func):
    """Scan for webserver

    Args:
        func (Any): function that raturn a subdomain
    """
    def wrapper(*args, **kwargs):
        value = func(*args, **kwargs)
        config = Config()
        if not config.scanning_modules:
            return ModuleStatus.ABORT
        
        
        try:
            cache_singleton = Cache()
            cached_result = cache_singleton.check_if_ip_already_found_and_return_result(ip=value.get('ip'))
            cached_uri = cache_singleton.check_if_uri_already_found_and_return_result(value.get('uri'))
            if 80 in cached_result.get('open_ports'):
                if cached_uri:
                    loader = Loader(f"{Y}       [->] Webserver Scanning...{Y}").start()
                    loader.end = f"{Y}       [*] Webserver Scanning{Y}{G} [DONE]{G}"
                else:
                    cache_singleton.add_subdomain_uri(value.get('uri'))
                webserver_scanning = HttpClientWebserverScanningAdapter()
                webserver_scanning.target_uri = value.get('ip')
                w_s = webserver_scanning.run()
                loader.stop()
                return w_s
            elif cached_uri:
                loader = Loader('').start()
                loader.end = f"{Y}       [*] Webserver Scanning (No port 80){Y}{G} [SKIPED]{G}"
                loader.stop()
        except:
            pass
        
        return []
    return wrapper


try:
    from dosuby.infrastructure.libs.CrtSearch.crt_search import CrtSearch
except ImportError:
    print("No module named 'search_engines' found")
    
from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
from time import sleep

class CrtAdapter(SubdomainEnumerator):
    
    def __init__(self) -> None:
        """This class will manage the enumeration logic using the CrtSearch

        """
        super().__init__()
        self.engine = CrtSearch()
    
    @property
    def queries(self):
        """Default CrtAdapter queries

        Returns:
            list: returns a list of CrtSearch queries to enumerate subdomains\n
            for a given target uri
        """
        return [
            f'{self.target_uri}'
        ]
    
    def add_dork_queries(self, query: str) -> None:
        """Add a new CrtSeach query
        """
        return super().add_dork_queries(query)
    

    def _process(self, query):
        for page in self.engine.search(query=query):
            yield page.find_all("td")
        


from dosuby.src.interfaces.security_enumerator_service import SecurityEnumeratorService
from dosuby.src.core.application.use_cases.ports_enumeration_use_case import PortsEnumerationUseCase
from dosuby.src.core.domain.target import Target
from dosuby.src.adapter.ports_scanning.socket_port_scanning_adapter import SocketPortScanningAdapter
from dosuby.src.interfaces.success_response import SuccessResponse

from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.strategies.security_enumeration.cli.socket_ports_cli_enumeration_strategy import SocketPortsCliEnumerationStrategy

class SocketPortScanningCliService(SecurityEnumeratorService):
    
 
    def __init__(self, enumeration_strategy=SocketPortsCliEnumerationStrategy(), success_response=SuccessResponse()) -> None:
        super().__init__(enumeration_strategy, success_response)
    
    
    def build_enumerator(self, target: Target) -> list:
        socket_port_scanning_adapter = SocketPortScanningAdapter()
        socket_port_scanning_usecase = PortsEnumerationUseCase(security_enumerator=socket_port_scanning_adapter)
        result = socket_port_scanning_usecase.execute(target=target)
        return result
    
    def process_enumerator(self, result) -> SuccessResponse:
        return self.enumeration_strategy.enumeration_process(ports=result, success_response=self.success_response)     
                                                            


try:
    from urllib.parse import urlparse
except ImportError:
    print("No module named 'urllib' found")

from dosuby.src.interfaces.domain_serializer import DomainSerializer

class ExtractUriSerializer(DomainSerializer):
    
    def serialize(uri: str=''):
        parsed_uri = urlparse(uri)
        parsed_uri = parsed_uri.netloc 
        if parsed_uri == '':
            return uri
        return parsed_uri



import pytest
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.domain.target import Target
from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer



# def test_success_response_has_global_final_result():
    
#     target_1 = Target(target_uri='domain.com', subdomain_serializer=ExtractUriSerializer)
#     success_response_1 = SuccessResponse()
#     success_response_1.set_target(target=target_1)
    
#     success_response_1.target.add_subdomain('sub1.domain.com')
#     success_response_1.target.add_subdomain('sub2.domain.com')
    
    
#     target_2 = Target(target_uri='domain.com', subdomain_serializer=ExtractUriSerializer)
#     success_response_2 = SuccessResponse()
#     success_response_2.set_target(target=target_2)
    
#     success_response_2.target.add_subdomain('sub3.domain.com')
#     success_response_2.target.add_subdomain('sub4.domain.com')
    
#     subdomains = success_response_2.get_target_subdomains()
    
    
#     assert target_1 is target_2
    
#     assert subdomains == ['sub1.domain.com','sub2.domain.com','sub3.domain.com','sub4.domain.com']


from dosuby.src.core.application.response.cli.failed_response_builder import FailureResponseBuilder
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException

def test_failed_build_invalid_request_exception_object():
    
    ex = InvalidTargetException({'parameter': 'uri', 'message': 'invalid'})
    
    error = FailureResponseBuilder().build_invalid_request_exception_object(ex)
    
    print(error.get_response())


from dosuby.src.interfaces.subdomain_enumerator_service import SubdomainEnumeratorService
from dosuby.src.core.application.use_cases.crt_enumeration_use_case import CrtEnumerationUseCase
from dosuby.src.core.domain.target import Target
from dosuby.src.adapter.crt.crt_adapter import CrtAdapter
from dosuby.src.interfaces.success_response import SuccessResponse

from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.strategies.crt.cli.crt_search_cli_enumeration_strategy import CrtSearchCliEnumerationStrategy

class CrtCliService(SubdomainEnumeratorService):
    
 
    def __init__(self, enumeration_strategy=CrtSearchCliEnumerationStrategy(), success_response=SuccessResponse()) -> None:
        super().__init__(enumeration_strategy, success_response)
    
    
    def build_enumerator(self, target: Target):
        crt_adapter = CrtAdapter()
        target_crt_usecase = CrtEnumerationUseCase(crt=crt_adapter)
        result = target_crt_usecase.execute(target=target)
        return result
    
    def process_enumerator(self, result) -> SuccessResponse:
        return self.enumeration_strategy.enumeration_process(rows=result, success_response=self.success_response)     
                                                            


import requests
from bs4 import BeautifulSoup

class CrtSearch():
    
    
    
    def search(self, query: str):
        base_url="https://crt.sh/?q={}".format(query)
        response = requests.get(base_url)
        html_content = response.content

        # Parse the HTML content using Beautiful Soup
        soup = BeautifulSoup(html_content, "html.parser")

        # Find the table in the HTML using the table tag and its attributes
        tables = soup.find_all("table")
        table = tables[1]

        # Find all the rows in the table (using the tr tag)
        rows = table.find_all("tr")
        
        for row in rows:
            yield row


from dataclasses import dataclass, field
from dosuby.src.interfaces.singleton import Singleton
from dosuby.src.interfaces.domain_serializer import DomainSerializer
from dosuby.src.core.application.input_dtos.target_input_dto import TargetInputDTO
from .subdomain import Subdomain
from typing import List
from dosuby.src.core.application.decorators.enumeration_decorators import *
from dosuby.src.core.application.decorators.loggers_decorators import *
from .config import Config

@dataclass
class Target(metaclass=Singleton):
    """Target Singleton

    Args:
        metaclass (Singleton): Defaults to Singleton.

    """
    
    target_uri: str
    _subdomain: Subdomain = field(init=False)
    _subdomains: List[Subdomain] = field(init=False,default_factory=list)
    config: Config = field(init=False)
    subdomain_serializer: DomainSerializer = None
    skip_logging: bool = field(init=False, default=False)
    _subdomain_count: int = field(init=False, default=0)
    
    
    def __post_init__(self):
        self.target_uri = TargetInputDTO(uri=self.target_uri)
        self.config = Config()
    
    @property
    def subdomain(self) -> Subdomain:
        return self._subdomain
    
    @subdomain.setter
    def subdomain(self, value: str) -> None:
        self._subdomain = Subdomain(
                                subdomain_uri=value, 
                                subdomain_serializer=self.subdomain_serializer
                            )
    
    @property
    def subdomains(self) -> list:
        return self._subdomains
    
    @subdomains.setter
    @cache_subdomain
    def subdomains(self, value):
        self._subdomains.append(value)
    
    
   
    
    
    @info_subdomain_found('skip_logging')
    def add_subdomain(self, subdomain: str) -> bool:
        self.subdomain = subdomain
        
        if (
            not any(sub == self.subdomain for sub in self.subdomains) 
            and self.target_uri.check_if_result_is_accurate(self.subdomain.subdomain_uri)
        ):
            self.subdomains = self._subdomain
            self._subdomain_count = self._subdomain_count + 1
            return True
        
        return False
    
    @property
    def subdomains_count(self):
        return self._subdomain_count
    
    def subdomain_count_init(self):
        self._subdomain_count = 0
        
    
    
    @save_enumeration_report
    def get_target_intel(self):
       if self.config.scanning_modules:
           sub_dict = lambda s: {
                                    'subdomain_uri': s.subdomain_uri,
                                    'subdomain_ip': s.subdomain_ip,
                                    'subdomain_hostname': s.subdomain_hostname,
                                    'subdomain_open_ports': s.open_ports,
                                    'subdomian_cms': s.cms,
                                    'subdomain_webserver': s.webserver
                                }
       else:
           sub_dict = lambda s: {
                                    'subdomain_uri': s.subdomain_uri,
                                    'subdomain_ip': s.subdomain_ip,
                                }
           
       
       return [(sub_dict)(sub) for sub in self.subdomains]
    
    


import logging
from dosuby.src.core.domain.config import Config
from dosuby.src.core.domain.cache import Cache
from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer
import socket
from dosuby.infrastructure.libs.loader import Loader

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow

LOADER = Loader()

def enumerating(func):
    def wrapper(*args, **kwargs):
        while True:
            global LOADER
            LOADER.stop()
            info = f"{C}[*]  Enumerating{C}"
            LOADER = Loader(info, end=f"{info} ->{W} [Done] {W}")
            LOADER.start()
            result = func(*args, **kwargs)
            return result
    return wrapper

def info_logger(message: str):
    def decorator(func):
        def wrapper(*args, **kwargs):
            # logging.info(f"{C}[*]  {message}{C}")
            
            # global LOADER
            # LOADER.stop()
            info = f"{C}[*]  {message}{C}"
            # LOADER = Loader(info, end=f"{info} ->{W} [Done] {W}")
            # LOADER.start()
            logging.info(info)
            return func(*args, **kwargs)
        return wrapper
    return decorator

def info_logger_attribute(attr_name):
    def decorator(func):
        def wrapper(*args, **kwargs):
            message = getattr(args[0], attr_name)
            logging.info(f"{C}[*]  {message}{C}")
            return func(*args, **kwargs)
        return wrapper
    return decorator

# def info_ip_found(attr_name):
#     def decorator(func):
#         def wrapper(*args, **kwargs):
#             skip = getattr(args[0], attr_name)
#             value = func(*args, **kwargs)
            
#             if not skip:
#                 if value:
#                     logging.info(f"[*]  IP found: {value}")
#                 else:
#                     logging.warning("[!]    Could not found IP address")
            
#             return value
        
#         return wrapper
#     return decorator

def info_subdomain_found(attr_name):
    def decorator(func):
        def wrapper(*args, **kwargs):
            skip = getattr(args[0], attr_name)
            value = func(*args, **kwargs)
            if not skip:
                if value:
                    config = Config()
                    cache = Cache()
                    sub_ser = ExtractUriSerializer.serialize(uri=args[1])
                    if not config.scanning_modules:
                        
                        ip = None
                        
                        try:
                            ip = socket.gethostbyname(sub_ser)
                        except:
                            pass
                        
                        if ip:
                            info = f"{G}[+]  {sub_ser} --> {ip}{G}"
                            logging.info(info)
                        else:
                            info = f"{Y}[!]  {sub_ser} [Could not find IP address]{Y}"
                            logging.info(info)
                            
                    # else:
                        # global LOADER
                        # LOADER.stop()
                        
                        # info = f"[*]{C}  Subdomain found {sub_ser}. Scanning for [open ports, CMS, WebServer]...{C}"
                        # LOADER = Loader(info, end=f"{info} ->{W} [Done] {W}")
                        # LOADER.start()
                        # logging.info(f"[*]{C}  Subdomain found {sub_ser}{C}")
                    
                    cache.cached_enumeration_result_count = cache.cached_enumeration_result_count + 1
            return value
        
        return wrapper
    return decorator


# def info_port_scanning(ports):
#     global LOADER
#     LOADER.stop()
#     while ports == 'EMPTY':
#         info = f"{C}[*]  Start port scanning{C}"
#         LOADER = Loader(info)
#         LOADER.start()
        
#     LOADER.end = f"{C}[*]  Port scanning{C} -> {G} [Done] {G}"
#     LOADER.stop()
    
#     return ports

# def info_cms_scanning(attr_name):
#     def decorator(func):
#         def wrapper(*args, **kwargs):
#             skip = getattr(args[0], attr_name)
#             value = func(*args, **kwargs)
#             if not skip:
#                 if value:
#                     global LOADER
#                     LOADER.stop()
#                     info = f"{C}[*]  Scanning for CMS{C}"
#                     LOADER = Loader(info, end=f"{info} ->{W} [Done] {W}")
#                     LOADER.start()
#             return value
        
#         return wrapper
#     return decorator

def log_subdomain_info(subdomain_uri):
    config = Config()
    logging.info(f"[*]{C}  ==> {subdomain_uri}{C}")
    # if config.scanning_modules:
    #     cache_singleton = Cache()
    #     if subdomain_uri not in cache_singleton.tmp_subdomains:
    #         cache_singleton.add_subdomain_uri(subdomain_uri)
    #         logging.info(f"[*]{C}  Subdomain found {subdomain_uri}{C}")




class InvalidTargetException(Exception):
    
    def __init__(self,error={}, *args: object) -> None:
        """This class with customize the input params exceptions

        Args:
            error (dict, optional): error value {'parameter': 'param_example', 'message': 'exception message example'}. Defaults to {}.
        """
        
        super().__init__(*args)
        self._errors = []
        if bool(error): self._errors.append(error)
    
    
    def add_error(self, parameter: str, message: str):
        """Append new errors

        Args:
            parameter (str): parameter name subject of the error
            message (str): exception error message
        """
        self._errors.append({'parameter': parameter, 'message': message})
    
    @property
    def errors(self):
        return self._errors
    
    
    def __bool__(self):
        return len(self.errors) > 0

    def __str__(self) -> str:
        # building a custom error message
        if len(self.errors) < 1: return self
        err_m = ''
        for err in self.errors:
            err_m = err_m+"{0}: {1}\n".format(err['parameter'], err['message'])
        
        return err_m[:-1]
    


from abc import ABC, abstractmethod
from dosuby.src.core.application.enums.cli_response_type_enum import CliResponseTypeEnums

class ResponseBuilder(ABC):
    """
    This is a builder interface to specifie methods for creating
    the different parts of the Response object
    """
    
    @staticmethod
    @abstractmethod
    def set_type(value=None):
        pass
    
    @staticmethod
    @abstractmethod
    def set_value(value=None):
        pass
    
        
    @staticmethod
    @abstractmethod
    def set_response_message(message_value=''):
        pass
    
    @staticmethod
    @abstractmethod
    def build_response():
        pass
    
    @staticmethod
    @abstractmethod
    def set_response_message_and_build(message_value=''):
        pass
    
    @staticmethod
    @abstractmethod
    def _set_status_code(resp_type: CliResponseTypeEnums):
        pass


from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator
import requests

class HackertargetAdapter(SubdomainEnumerator):
    
    def __init__(self) -> None:
        """Manage the enumeration logic using Hackertarget

        """
        super().__init__()
        self.base_url = "https://api.hackertarget.com/hostsearch/"
        self.headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36"}
    
    @property
    def queries(self):
        """Default HackertargetAdapter queries

        Returns:
            list: returns a list of Hackertarget  queries to enumerate subdomains\n
            for a given target uri
        """
        return {"q": self.target_uri}
    
    def add_dork_queries(self, query: str) -> None:
        """Add a new Anubis dork query

        Args:
            query (str): should be a google dork\nex: f'inurl:{target_uri}'

        Returns:
            _type_: None
        """
        return super().add_dork_queries(query)
    

    def _process(self, query):
        # write your process logic here
        response = requests.get(self.base_url, headers=self.headers, params=self.queries)
        if response.status_code == 200:
            try:
                line = response.text.split('\n')
                # Find the <pre> tag
                for l in line:
                    subdomain = l.split(',')[0]
                    yield subdomain
            except:
                pass
        
        return []
            
        


from abc import ABC, abstractmethod

class SecurityEnumeration(ABC):
    
    def __init__(self) -> None:
        """This class will manage the security enumeration logic

        """
        super().__init__()
        self._target_uri = ''
        self.logger_message = 'SecurityEnumeration'
    
    
    @property
    def target_uri(self):
        return self._target_uri
    
    @target_uri.setter
    def target_uri(self, value):
        self._target_uri = value
        
    
    @abstractmethod
    def run(self):
        """This method will run the security enumeration process

        """
        pass
    
    
    @abstractmethod
    def _process(self, **kwargs):
        """This will run the dork enumeration logic
        """
        pass


from abc import ABC, abstractmethod
from dosuby.src.core.domain.target import Target
from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.interfaces.enumeration_strategy import EnumerationStrategy


class SubdomainEnumeratorService(ABC):
    """Abstract class that should be implemented for enumerators services
    """
    
    
    def __init__(self,enumeration_strategy: EnumerationStrategy, success_response: SuccessResponse) -> None:
        """Enumerator service

        Args:
            success_response (SuccessResponse, optional): this attributes will have the service success result. Defaults to SuccessResponse().
        """
        super().__init__()
        self.success_response = success_response
        self.enumeration_strategy = enumeration_strategy
    
    
    def read(self, uri: str):
        """Read the dorks data to get the domain subdomains using dorks
        
        Args:
            uri (str): uri target
        """
        try:
            # probably this should be a singletone
            target = self._get_target_method(uri=uri)
        except InvalidTargetException as ex:
            raise ex
                                    
            # return failed_response
        
        try:
            
            
            self.success_response.set_target(target=target)
            
            result = self.build_enumerator(target=target)
            
            return self.process_enumerator(result)
            
        
        except Exception as ex:
            raise ex
        
    
    @abstractmethod
    def build_enumerator(self, target: Target):
        """This method will build the enumerator to use. example:\n
        google_dork = GoogleDorksAdapter()\n
        target_google_dork_usecase = DorksEnumerationUseCase(dork=google_dork)\n
        result = target_google_dork_usecase.execute(target=target)\n
        return result\n

        Args:
            target (Target): target to enumerate

        Raises:
            NotImplementedError: This method should be used for any enumerator service
        """
        raise NotImplementedError
    
    @abstractmethod
    def process_enumerator(self, result) -> SuccessResponse:
        """This method will hold the logic of enumeration process for each service\n
        this method should use the self.success_response and return it after the process is over.

        Args:
            result (Generator): the result should be a generator

        Raises:
            NotImplementedError: This method should be used for any enumerator service
        """
        raise NotImplementedError
                    
    def _get_target_method(self, uri: str) -> Target:
        """private method to create a target singleton object

        Args:
            uri (str): domain to enumerate

        Returns:
            Target
        """
        return Target(target_uri=uri, subdomain_serializer=ExtractUriSerializer)
    
    


from dosuby.src.core.domain.target import Target
from dosuby.src.interfaces.subdomain_enumerator import SubdomainEnumerator

class VirustotalEnumerationUseCase:
    """This is the Virustotal Enumeration Use Case.\n
    it will be used to enumerate subdomains using virustotal
    """
    
    def __init__(self, virustotal: SubdomainEnumerator) -> None:
        """Inject an enumerator module

        Args:
            virustotal (SubdomainEnumerator): subdomain enumerator
        """
        
        if not isinstance(virustotal, SubdomainEnumerator):
            raise ValueError(f'virustotal param should be a SubdomainEnumerator instance, not a {type(virustotal)}')
        
        self._virustotal = virustotal
    
    def execute(self, target: Target):
        """Execute method will run the enumeration process

        Args:
            target (Target): url or domain to enumerate
        """
        
        if not isinstance(target, Target):
            raise ValueError(f'target param should be a Target instance, not a {type(target)}.')
        
        self._virustotal.target_uri = target.target_uri.uri
        
        return self._virustotal.run()
    


try:
    import http.client
except ImportError:
    print("HttpClientWebserverScanning: No module named 'http' found")

    
from dosuby.src.interfaces.webserver_enumeration_adapter import WebserverEnumerationAdapter

class HttpClientWebserverScanningAdapter(WebserverEnumerationAdapter):
    
    def __init__(self) -> None:
        """This class will manage the port enumeration logic using Sockets

        """
        super().__init__()
    
    
    def _process(self):
        conn = http.client.HTTPConnection(self.target_uri)
        conn.request("GET", "/")
        response = conn.getresponse()
        server_header = response.getheader("Server")
        if server_header:
            return server_header
        else:
            return "Unknown"
        



    



import pytest
from dosuby.src.core.domain.target import Target
from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer

# def test_init_target():
#     target = Target(target_uri='domain.test.com')
    
#     assert target.subdomains == []


# def test_if_target_is_a_singleton():
    
#     target_1 = Target(target_uri='domain.com')
#     target_2 = Target(target_uri='domain.com')
    
#     assert target_1 is target_2


# def test_add_subdomain():
#     target = Target(target_uri='domain.com', subdomain_serializer=ExtractUriSerializer)
#     target_subdomains_1 = 'test1.domain.com'
#     target_subdomains_2 = 'test2.domain.com'
    
#     target.add_subdomain(subdomain=target_subdomains_1)
#     target.add_subdomain(subdomain=target_subdomains_2)
    
#     assert target.subdomains == ['test1.domain.com', 'test2.domain.com']

# def test_adding_a_duplicated_subdomain():
#     target = Target(target_uri='domain.com')
#     target_subdomains_1 = 'test1.domain.com'
#     target_subdomains_2 = 'test2.domain.com'
    
#     target.add_subdomain(subdomain=target_subdomains_1)
#     target.add_subdomain(subdomain=target_subdomains_2)
    
#     assert target.add_subdomain(subdomain=target_subdomains_2) is False

# def test_adding_a_new_subdomain():
#     target = Target(target_uri='domain.com')
#     target_subdomains_1 = 'test1.domain.com'
#     target_subdomains_2 = 'test2.domain.com'
#     target_subdomains_3 = 'test3.domain.com'
    
#     target.add_subdomain(subdomain=target_subdomains_1)
#     target.add_subdomain(subdomain=target_subdomains_2)
    
#     assert target.add_subdomain(subdomain=target_subdomains_3) is True


# def test_check_if_result_is_accurate():
#     target = Target(target_uri='domain.com')
#     target_subdomains_1 = 'test1.domain.com'
#     target_subdomains_2 = 'test2.domainn.com'
#     target_subdomains_3 = 'test3.domain.com'
    
#     target.add_subdomain(subdomain=target_subdomains_1)
#     target.add_subdomain(subdomain=target_subdomains_2)
#     target.add_subdomain(subdomain=target_subdomains_3)
    
#     subdomains = target.subdomains
    
#     assert  len(subdomains) == 2
    




from dataclasses import dataclass, field
from dosuby.src.interfaces.singleton import Singleton


@dataclass
class EnumerationReporte(metaclass=Singleton):
    """Cache results to use in real time execution
    """

    report_subdomains: list = field(init=False, default_factory=list)

    def add(self, subdomain: list) -> None:
        self.report_subdomains.extend(subdomain)



import shutil
import os

"""This will create a new enumeration module object
"""
class DosubyModule:


    def __init__(self,class_name: str):
        self.class_name = class_name.lower()
        self.USE_CASE_TEMPLATE = "src/utils/module_template/use_case/template.txt"
        self.USE_CASE_PATH = "src/core/application/use_cases/"
        
        self.STRATEGY_TEMPLATE = "src/utils/module_template/strategy/cli/template.txt"
        self.STRATEGY_PATH = f"src/core/application/strategies/{class_name}/cli/"
        
        self.ADAPTER_TEMPLATE = "src/utils/module_template/adapter/template.txt"
        self.ADAPTER_PATH = f"src/adapter/{class_name}/"
        
        self.HANDLER_TEMPLATE = "src/utils/module_template/handlers/cli/template.txt"
        self.HANDLER_PATH = f"src/handlers/cli/"
        
        self.SERVICE_TEMPLATE = "src/utils/module_template/services/cli/template.txt"
        self.SERVICE_PATH = f"src/services/{class_name}/cli/"
    
    def create(self):
        # read class name
        class_name = self.class_name

        if not class_name.isalpha():
            print(f"[x] Invalid class name '{class_name}'")
            return False

        # Start by creating a new UseCase
        with open(self.USE_CASE_TEMPLATE, 'r') as f:
            use_case_template = f.read()
            
        with open(self.STRATEGY_TEMPLATE, 'r') as f:
            strategy_template = f.read()
        
        with open(self.ADAPTER_TEMPLATE, 'r') as f:
            adapter_template = f.read()
            
        with open(self.HANDLER_TEMPLATE, 'r') as f:
            handler_template = f.read()
        
        with open(self.SERVICE_TEMPLATE, 'r') as f:
            service_template = f.read()


        # Replace the class name with the provided argument
        use_case = use_case_template.replace("{{class}}", class_name)\
                                    .replace("{{Class}}", class_name.capitalize())
                                    
        strategy = strategy_template.replace("{{class}}", class_name)\
                                .replace("{{Class}}", class_name.capitalize())
        
        adapter = adapter_template.replace("{{class}}", class_name)\
                                .replace("{{Class}}", class_name.capitalize())

        handler = handler_template.replace("{{class}}", class_name)\
                            .replace("{{Class}}", class_name.capitalize())
        
        service = service_template.replace("{{class}}", class_name)\
                            .replace("{{Class}}", class_name.capitalize())

        # Write the resulting use_case to a new file
        path = f'{self.USE_CASE_PATH}{class_name}_enumeration_use_case.py'
        with open(path, 'w') as f:
            f.write(use_case)
            print(f"[+] Creating '{path}' -> UseCase")
            
        # creating the strategy
        # Create the directory path if it does not exist
        os.makedirs(os.path.dirname(self.STRATEGY_PATH), exist_ok=True)
        # Write the resulting strategy to a new file
        path = f'{self.STRATEGY_PATH}{class_name}_cli_enumeration_strategy.py'
        with open(path, 'w') as f:
            f.write(strategy)
            print(f"[+] Creating '{path}' -> Strategy")
            self.create_init(self.STRATEGY_PATH.replace('/cli/','/'))

        os.makedirs(os.path.dirname(self.ADAPTER_PATH), exist_ok=True)
        path = f'{self.ADAPTER_PATH}{class_name}_adapter.py'
        with open(path, 'w') as f:
            f.write(adapter)
            print(f"[+] Creating '{path}' -> Adapter")
            self.create_init(self.ADAPTER_PATH)
        
        os.makedirs(os.path.dirname(self.SERVICE_PATH), exist_ok=True)
        # Write the resulting service to a new file
        path = f'{self.SERVICE_PATH}{class_name}_cli_service.py'
        with open(path, 'w') as f:
            f.write(service)
            print(f"[+] Creating '{path}' -> Service")
            self.create_init(self.SERVICE_PATH.replace('/cli/','/'))
        
        os.makedirs(os.path.dirname(self.HANDLER_PATH), exist_ok=True)
        path = f'{self.HANDLER_PATH}{class_name}_handler.py'
        with open(path, 'w') as f:
            f.write(handler)
            print(f"[+] Creating '{path}' -> Handler")

    def create_init(self, path: str):
        with open(f"{path}__init__.py", 'w') as f:
            f.write('')
    
    
    def delete(self):
        
        try:
            use_case_path = f'{self.USE_CASE_PATH}{self.class_name}_enumeration_use_case.py'
            strategy_path = f'src/core/application/strategies/{self.class_name}/'
            adapter_path = self.ADAPTER_PATH
            handler_path = f'{self.HANDLER_PATH}{self.class_name}_handler.py'
            service_path = f'src/services/{self.class_name}/'

            os.remove(use_case_path)
            print(f"[-] {use_case_path} -> Use case removed")
            
            shutil.rmtree(strategy_path)
            print(f"[-] {strategy_path} -> Strategy removed")
            
            shutil.rmtree(adapter_path)
            print(f"[-] {adapter_path} -> Adapter removed")
            
            os.remove(handler_path)
            print(f"[-] {handler_path} -> Handler removed")
            
            shutil.rmtree(service_path)
            print(f"[-] {service_path} -> Service removed")
            
        except Exception as e:
            print("[x] Error: {e}".format(e=e))
            print(f"[!] Probably the module '{self.class_name}' doesn't exist")
    
    
    @staticmethod
    def list_modules():
        path = 'src/handlers/cli/'
        files = os.listdir(path)
        i = 0
        for file in files:
            file = file.replace('_handler.py','')
            if file not in ['__init__.py', '__pycache__', 'handler_cli.py']:
                i = i + 1
                print(f"-> {file}")
                
        print(f"[Total: {i}]")


from dataclasses import dataclass
from dosuby.src.core.application.validators.validate_target_input import ValidateTargetInput
from dosuby.src.core.application.exceptions.invalid_target_input_exception import InvalidTargetException

@dataclass
class TargetInputDTO:
    """This dataclass with manage the target inputs and parameters\n
        like the uri we would like to enumerat, what search engine dork we like to use...
    """
    uri: str

    def __post_init__(self):
        self.uri = ValidateTargetInput.extract_domain(target_uri=self.uri)
        if self.uri is False:
            raise InvalidTargetException(error={'parameter': 'uri', 'message': 'Invalid target uri'})
    
    def check_if_result_is_accurate(self, subdomain: str):
        """This method will check if the subdomain found is really belongs to the target domain or not
        

        Args:
            subdomain (str): the enumerated subdomain
        """
        return (
            self.uri
            in subdomain
            and '*' not in subdomain
            and f"www.{self.uri}" != subdomain
            and self.uri != subdomain
            and subdomain.endswith(f".{self.uri}")
        )
        


try:
    from socket import *
except ImportError:
    print("SocketPortScanningAdapter: No module named 'socket' found")

try:
    from threading import *
except ImportError:
    print("SocketPortScanningAdapter: No module named 'threading' found")

    
from dosuby.src.interfaces.ports_enumeration_adapter import PortEnumerationAdapter

class SocketPortScanningAdapter(PortEnumerationAdapter):
    
    def __init__(self) -> None:
        """This class will manage the port enumeration logic using Sockets

        """
        super().__init__()
        
    
    
    def _process(self, ip, port):
        try:
            # creates a socket object to connect to that port on the target host.
            server_sock = socket(AF_INET, SOCK_STREAM)
            server_sock.settimeout(1.5)
            # If the connection is successful, the method yields the open port number.
            result = server_sock.connect((ip, port))
            yield port
        except:
            # If there's no connection return nothing
            return
        finally:
            # close socket connection
            server_sock.close()
    
    def _scan(self, ip):
    # The _scan method iterates through the range of ports to scan 
    # and calls the _process method on each port.
    # If an open port is found, it yields the open port number.
        for port in self.ports:
            for open_port in self._process(ip, port):
                yield open_port
    
    def run(self):
        # get ip of target domain
        target_ip = gethostbyname(self.target_uri)
        # Create an empty list to store the threads
        threads = []
        # Loop through the range of ports to scan
        for port in self.ports:
            # Create a new thread for each port
            thread = Thread(target=self._process, args=(target_ip, port))
            # Add the thread to the list
            threads.append(thread)
            # Start the thread
            thread.start()
        
        # Wait for all threads to finish
        for thread in threads:
            thread.join()
        
        open_ports = list(self._scan(target_ip))
        return open_ports
    


    



from abc import ABC, abstractmethod
from dosuby.src.interfaces.enumeration_handler import EnumerationHandler
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.response.cli.failed_response_builder import FailureResponseBuilder
from dosuby.src.core.domain.cache import Cache


class Handler(EnumerationHandler):
    """
    The Handler interface declares a method for building the chain of subdomain handlers.\n
    It builds a way for creating chained enumerators that enumerat for subdomains is different ways\n
    and return a one final result to the user
    
    """
    
    def __init__(self, next_handler: EnumerationHandler=None) -> None:
        self._next_handler = next_handler
        self.cache = Cache()

    
    def handle(self, uri: str, success_response: SuccessResponse=SuccessResponse()):
        """This method will set the next handler

        Args:
            uri (str): domain target to handle
            success_response (SuccessResponse): success response (will be passed to the next handler)

        Returns:
            Handler: Next handler
        """
        try:
            
            # process to put before the handler
            self.pre_handler_process()
            self._handler_process(uri=uri, success_response=success_response)
            
        except Exception as ex:
            failed_response = FailureResponseBuilder().build_system_error(ex)
            return failed_response.get_response()
            
    
    def run_service(self, uri: str, success_response: SuccessResponse):
        return super().run_service(uri, success_response)

    
    def pre_handler_process(self, **kwargs):
        """This method will be used for specific pre handlers proccessing like displaying a handler name in the console
        """
        pass
    
    
    def _handler_process(self, uri: str, success_response: SuccessResponse):
        self.cache.cached_enumeration_result_count = 0
        service_response = self.run_service(uri=uri, success_response=success_response)
        if type(service_response) is SuccessResponse and self._next_handler:
            return self._next_handler.handle(uri,service_response)
        


from abc import ABC, abstractmethod
from dosuby.src.core.domain.config import Config

class ReportRepo(ABC):
    
    def __init__(self) -> None:
        super().__init__()
        self.config = Config()
    
    @abstractmethod
    def read_report(self):
        NotImplementedError("read_report is not implemented")


from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='dosuby',
    version='1.2.0',
    packages=find_packages(),
    install_requires=[
        'inquirer',  # Add other dependencies as needed
        'art',
    ],
    entry_points={
        'console_scripts': [
            'dosuby=dosuby.cli:main',  # CLI entry point
        ],
    },
    author='deidax',
    author_email='deidaxtech@gmail.com',
    description='A tool for enumerating subdomains of a parent domain',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/deidax/dosuby',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Security',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Natural Language :: English',
    ],
    python_requires='>=3.10',
    keywords='subdomain enumeration security network',
    license='MIT',
)


from dosuby.src.interfaces.security_enumeration import SecurityEnumeration



class PortEnumerationAdapter(SecurityEnumeration):

    def __init__(self) -> None:
        """This class will manage the port enumeration logic using the Sockets

        """
        super().__init__()
        self._ports = [21, 22, 23, 25, 53, 80, 110, 135, 139, 443, 3306, 5432]
        self.logger_message = f"Starting port scanning"
    
    @property
    def ports(self):
        return self._ports
    
    @ports.setter
    def ports(self, value):
        self._ports.append(value)
    
    
    def run(self):
        yield self._process()
    
    
    def _process(self, **kwargs):
        return super()._process(**kwargs)        



import inquirer
import re
from typing import List
import logging
from dosuby.src import *
from art import *

def main():
    """Main function for the interactive CLI."""
    tprint(text="Dosuby")
    with open('dosuby/version.txt', 'r') as f:
        version = f.read().strip()
    print('version:', version)
    
    
    logging.basicConfig(
                level=logging.INFO, 
                format="%(message)s",
                datefmt="%Y-%m-%d %H:%M:%S"
            )
    config = Config()
    config.scanning_modules = False
    
    print('\n')
    
    regex = r"^[a-z]{2,}\.[a-z]{2,}$"
    handlers = [
        {'id': '1','name':'Google', 'value': GoogleDorkHandler},
        {'id': '2','name':'Yahoo', 'value': YahooDorkHandler},
        # {'id': '3','name':'Duckduckgo', 'value': DuckduckgoDorkHandler},
        {'id': '4','name':'Brave', 'value': BraveDorkHandler},
        {'id': '5','name':'Aol', 'value': AolDorkHandler},
        {'id': '6','name':'Bing', 'value': BingDorkHandler},
        {'id': '7','name':'Ask', 'value': AskDorkHandler},
        {'id': '8','name':'Crt Search', 'value': CrtSearchHandler},
        {'id': '9','name':'Anubis', 'value': AnubisHandler},
        {'id': '10','name':'AlientVault', 'value': AlientvaultHandler},
        {'id': '11','name':'HackerTarget', 'value': HackertargetHandler},
        {'id': '12','name':'WaybackMachine', 'value': WaybackmachineHandler},
        {'id': '13','name':'VirusTotal', 'value': VirustotalHandler}
    ]

    # Extract the 'name' keys from the list of dictionaries
    handlers_choices = [handler['name'] for handler in handlers]
   
    questions = [
    inquirer.Text('TLD', message="What's the domain to enumerate",
                       validate=lambda _, x: re.match(regex, x)
               ),
    inquirer.Checkbox('handlers',
                    message="Choose methods to use for the enumeration (to use all options just press ENTER without any selections. to select a specific options press SPACE and then ENTER)",
                    choices=handlers_choices,
                    ),
    inquirer.Confirm("scanning_modules", message="Do you want to scan subdomains?",)
    ]
    
    answers = inquirer.prompt(questions)
    
    target_uri = answers.get('TLD')
    selected_handlers = answers.get('handlers')
    config.scanning_modules = answers.get('scanning_modules')
    
    found_handlers: List[dict] = []

    for selected_handler in selected_handlers:
        if selected_handler in handlers_choices:
            for handler in handlers:
                if selected_handler == handler['name']:
                    found_handlers.append(handler)
    
    handlers_selected: List[Handler] = []
    if not found_handlers:
        all_handlers = handlers
        for h in all_handlers:
            handlers_selected.append(h.get('value'))
        
    try:
        handlers_selected_objects: List[Handler] = []
        
        for handler in found_handlers:
            handler_class_name = handler.get('value')
            handlers_selected.append(handler_class_name)
        
        for handler_selected_class_name in reversed(handlers_selected):
            handler_object: Handler = None
            next_handler = None
            if handlers_selected_objects:
                next_selected_handler = handlers_selected_objects[-1]
                next_handler = next_selected_handler
                
            handler_object = handler_selected_class_name(next_handler=next_handler)
            handlers_selected_objects.append(handler_object)
        
        handlers_to_use = handlers_selected_objects[::-1]
        
        handler_starter = handlers_to_use[0]
        
        s = f"\n{'-'*20}[Starting...]{'-'*20}"
        print(s)
        handler_starter.handle(uri=target_uri)
        
        report = CliReportRepo()
        report.read_report()
        
    except Exception as ex:
        print(ex)
            


if __name__ == "__main__":
    main()


from dosuby.src.interfaces.enumeration_strategy import EnumerationStrategy
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.response.cli.success_response_builder import SuccessResponseBuilder


class AnubisCliEnumerationStrategy(EnumerationStrategy):
    
    
    def enumeration_process(self, subdomains_links, **kwargs) -> list:
        """_summary_

        Args:
            rows(generator): crt row  to process
            success_response (SuccessResponse): this will be used to get the final success response
        Returns:
            list: results list
        """
        success_response = kwargs.get('success_response')
        success_response_builder = SuccessResponseBuilder()
        success_response_builder.success_response = success_response
        # success response for each handler
        tmp_success_response = SuccessResponse()
        for subdomain in subdomains_links:
            for sub in subdomain:
                if success_response.target.add_subdomain(sub) is True:
                    tmp_success_response = success_response_builder.set_response_message_and_build('Subdomain Found!')
                    self.display_result(tmp_success_response.get_response())
            
        self.display_result_count(succes_response=tmp_success_response)
        
        return tmp_success_response


from dataclasses import dataclass, field
from dosuby.src.interfaces.singleton import Singleton

@dataclass
class Cache(metaclass=Singleton):
    """Cache ips to use in real time execution
    """
    
    #this will be used for display
    tmp_subdomains: list = field(init=False, default_factory=list) 
    cached_subdomains: list = field(init=False, default_factory=list)
    cached_enumeration_result_count: int = field(init=False, default=0)
    
    def add(self, subdomain: str) -> None:
        """Add a subdomain to the cache"""
        if subdomain not in self.cached_subdomains:
            self.cached_subdomains.append(subdomain)
    
    def add_subdomain_uri(self, subdomain: str) -> None:
        if subdomain not in self.tmp_subdomains:
            self.tmp_subdomains.append(subdomain)
    
    def check_if_ip_already_found_and_return_result(self, ip) -> dict | None:
        """Check if the given ip is already in the cache"""
        if self.cached_subdomains:
            result = list(filter(lambda s: s.get('ip') == ip, self.cached_subdomains))
            if len(result) > 0:
                return result[0]
        return []
    
    def check_if_uri_already_found_and_return_result(self, uri) -> dict | None:
        """Check if the given ip is already in the cache"""
        if self.tmp_subdomains:
            result = list(filter(lambda s: s == uri, self.tmp_subdomains))
            if len(result) > 0:
                return result[0]
        return []
    
    


from dosuby.src.interfaces.enumeration_strategy import EnumerationStrategy
from dosuby.src.core.application.response.cli.success_response_builder import SuccessResponseBuilder

class SocketPortsCliEnumerationStrategy(EnumerationStrategy):
    
    
    def enumeration_process(self, **kwargs) -> list:
        """_summary_

        Args:
            result(generator): result  to process
            success_response (SuccessResponse): this will be used to get the final success response
        Returns:
            list: results list
        """
        # success response for each handler
        ports = kwargs.get('ports')
       
        if ports:
            return ports
        return []


from abc import ABC, abstractmethod
from dosuby.src.core.domain.target import Target
from dosuby.src.serializers.extract_domain_serializer import ExtractUriSerializer


class SearchResponse(ABC):
    
    def __init__(self) -> None:
        """This abstract class manages the search response
        """
        self._res_type = None
        self._res_value = None
        self._target = None
        self._res_message = ''
        self._status_code = None
    
    @property
    def response_value(self):
        return self._res_value
    
    @response_value.setter
    def response_value(self, value):
        self._res_value = value
    
    @property
    def response_type(self):
        return self._res_type['label']
    
    @response_type.setter
    def response_type(self, value):
        self._res_type = value
        
    @property
    def target(self):
        return self._target
    
    @target.setter
    def target(self, value):
        self._target.add_subdomain(value)
    
    @property
    def response_message(self):
        return self._res_message
    
    @response_message.setter
    def response_message(self, value):
        self._res_message = value
    
    @property
    def status_code(self):
        return self._status_code
    @status_code.setter
    def status_code(self, value):
        self._status_code = value
    
    
    def __bool__(self):
        return self._res_type['value']
    
    @abstractmethod
    def get_response(self):
        """Return search results
        """
        pass
    
    @abstractmethod
    def set_target(self, target: Target):
        self._target = target


from dosuby.src.interfaces.response_builder import ResponseBuilder
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.enums.cli_response_type_enum import CliResponseTypeEnums

class SuccessResponseBuilder(ResponseBuilder):

    def __init__(self) -> None:
        """
        A builder instance should contain a blank response object
        which is used in further assembly
        """
        self._success_response = SuccessResponse()
        
    
    @property
    def success_response(self):
        return self._success_response
    
    @success_response.setter
    def success_response(self, value):
        self._success_response = value
    
    def set_type(self, value):
        """Success Response is set by default"""
        pass
    
    def set_value(self, value):
        self._success_response.response_value = value
        self._success_response.target.subdomain = value
        return self
    
    def set_response_message(self, message_value: str):
        self._success_response.response_message = message_value
        return self
    
    def build_response(self):
        return self._success_response
    
    def set_response_message_and_build(self, message_value):
        success_builder = self.set_response_message(message_value)
        return success_builder.build_response()
    
    def _set_status_code(resp_type: CliResponseTypeEnums):
        """Status code is set by default for Success Response"""
        pass


from dataclasses import dataclass, field
from dosuby.src.interfaces.domain_serializer import DomainSerializer
from dosuby.src.core.application.decorators.enumeration_decorators import *
from dosuby.src.core.application.decorators.loggers_decorators import *
from typing import Any

@dataclass
class Subdomain:
    
    subdomain_serializer: DomainSerializer
    subdomain_uri: str = field(init=True)
    open_ports: list = field(init=False, default_factory=list)
    cms: Any = field(init=False)
    webserver: Any = field(init=False)
    skip_logging: bool = field(init=False)
    skip_subdomain_login: bool = field(init=False, default=False)
    _subdomain_cms: Any = field(init=False, default='')
    _subdomain_uri: str = field(init=False, default='')
    _subdomain_ip: str = field(init=False, default='')
    _subdomain_hostname: str = field(init=False, default='')
    _subdomain_open_ports_from_uri: Any = field(init=False, default='')
    _subdomain_webserver_from_ip: Any = field(init=False, default='')
    _tmp_s_uri: str = field(init=False, default='')
    _tmp_s_ip: str = field(init=False, default='')
    
    
    @property
    def subdomain_uri(self) -> str:
        return self._subdomain_uri
    
    @subdomain_uri.setter
    def subdomain_uri(self, value: str) -> None:
        
        self.skip_logging = True
        
        if self.subdomain_serializer:
            
            
            self._subdomain_uri = self.subdomain_serializer.serialize(value)
            # assign subdomain_uri to get ip and open ports for the asseigned subdomain uri
            self._tmp_s_uri = self.subdomain_uri
            # log_subdomain_info(self._tmp_s_uri)
            self._subdomain_ip = self._tmp_s_uri
            self._tmp_s_ip = self.subdomain_ip
            
            self._subdomain_open_ports_from_uri = {'ip':self._tmp_s_ip,'uri': self._tmp_s_uri}
            
            
            self._subdomain_cms = {'ip':self._tmp_s_ip,'uri': self._tmp_s_uri}
            self._subdomain_webserver_from_ip = {'ip':self._tmp_s_ip,'uri': self._tmp_s_uri}
        else:
            self._subdomain_uri = value

        self.skip_logging = False
        
    
    @property
    @get_ip
    def subdomain_ip(self) -> str:
        return self._subdomain_ip
    
    
    @property
    @get_hostname
    @get_ip
    def subdomain_hostname(self) -> str:
        return self._subdomain_hostname
    
    @property
    @save_cms('cms')
    @scan_for_cms
    # @info_logger('Scanning for CMS')
    def subdomain_cms(self):
        return self._subdomain_cms
    
    
    @property
    @add_to_list('open_ports')
    @get_open_ports
    # @info_port_scanning('skip_logging')
    def subdomain_open_ports_from_uri(self) -> list:
        """Scan for open ports

        Returns:
            list: open ports list
        """
        return self._subdomain_open_ports_from_uri
    
    
    @property
    @save_webserver('webserver')
    @get_webserver
    def subdomain_webserver(self) -> str:
        """Scan for webserver

        Returns:
            list: webserver list
        """
        return self._subdomain_webserver_from_ip

    
    
    
    def __eq__(self, __o: object) -> bool:
        return self.subdomain_uri == __o.subdomain_uri
    
    
    def get_cached_data(self) -> dict:
        """Get the subdomain cached data

        Returns:
            dict: cached subdomain data
        """
        self.skip_logging = True
        
        cached_data = {
            'ip': self._tmp_s_ip,
            'uri': self._tmp_s_uri,
            'open_ports': self.subdomain_open_ports_from_uri,
        }
        
        self.skip_logging = False
        
        return cached_data
    
    


from dosuby.src.interfaces.enumeration_strategy import EnumerationStrategy
from dosuby.src.interfaces.success_response import SuccessResponse
from dosuby.src.core.application.response.cli.success_response_builder import SuccessResponseBuilder
from dosuby.src.core.application.decorators.loggers_decorators import info_logger

class CMSCliEnumerationStrategy(EnumerationStrategy):
    
    
    @info_logger('CMS Scanning')
    def enumeration_process(self, **kwargs) -> list:
        """_summary_

        Args:
            result(generator): result  to process
            success_response (SuccessResponse): this will be used to get the final success response
        Returns:
            list: results list
        """
        # success_response = kwargs.get('success_response')
        # success_response_builder = SuccessResponseBuilder()
        # success_response_builder.success_response = success_response
        # # success response for each handler
        # tmp_success_response = SuccessResponse()
        cms = kwargs.get('cms')
       
        # for port in list(next(ports)):
        #     print(port)
        #     print(next(port))
        #     print(next(port))

        #         print(p)
        # for port in ports:
        #     while True:
        #         try:
        #             p = next(port)
        #             print(p)
        #         except StopIteration:
        #             break
        
        # for row in rows:
        #     for cell in row:
        #         if len(cell) >= 5:
        #             sub = cell[4]
        #             if success_response.target.add_subdomain(sub.text) is True:
        #                 tmp_success_response = success_response_builder.set_response_message_and_build('Subdomain Found!')
        #                 print(tmp_success_response.get_response())
            
        # subdomains = tmp_success_response.get_target_subdomains()
        # print(subdomains)
        # if ports:
        #     return ports
        return cms